#!/usr/bin/env bash

if [ $# -lt 1 ]; then
  echo "usage: $0 DUNE_BASE_DIR"
  exit 1
fi

DUNE_BASE_DIR="$1"

mkdir -p ${DUNE_BASE_DIR}
cd ${DUNE_BASE_DIR}

CORE_MODULES=("dune-common" "dune-geometry" "dune-grid" "dune-istl" "dune-localfunctions")
STAGING_MODULES=("dune-functions" "dune-typetree" "dune-uggrid")
EXTENSIONS_MODULES=("dune-alugrid" "dune-spgrid" "dune-foamgrid")

for MOD in ${CORE_MODULES[*]}; do
  git clone https://gitlab.dune-project.org/core/${MOD}.git --branch releases/2.6 --single-branch
done

for MOD in ${STAGING_MODULES[*]}; do
  git clone https://gitlab.dune-project.org/staging/${MOD}.git --branch releases/2.6 --single-branch
done

for MOD in ${EXTENSIONS_MODULES[*]}; do
  git clone https://gitlab.dune-project.org/extensions/${MOD}.git --branch releases/2.6 --single-branch
done

echo ""
echo "=============================================================="
echo "To build all dune modules run the commands...                 "
echo "  export PATH=${DUNE_BASE_DIR}/dune-common/bin:\${PATH}"
echo "  DUNE_CONTROL_PATH=${DUNE_BASE_DIR} dunecontrol configure"
echo "  DUNE_CONTROL_PATH=${DUNE_BASE_DIR} dunecontrol make -j4 "
echo "=============================================================="