# FindMTL.cmake
#
# Finds the MTL4 library
#
# This will define the following variables
#
#    MTL_FOUND
#    MTL_INCLUDE_DIR
#    MTL_COMPILE_DEFINITIONS
#
# and the following imported targets
#
#     MTL::MTL
#
# Author: Simon Praetorius <simon.praetorius@tu-dresden.de>


mark_as_advanced(MTL_FOUND MTL_COMPILE_DEFINITIONS MTL_INCLUDE_DIR)

find_path(MTL_INCLUDE_DIR boost/numeric/mtl/mtl.hpp
  HINTS
    ${CMAKE_CURRENT_SOURCE_DIR}/../../libs/mtl4
    ${CMAKE_CURRENT_SOURCE_DIR}/../../include/mtl4
    ${MTL_DIR}/../../include
    $ENV{MTL_DIR}/../../include
    ENV EBROOTMTL
  PATHS
    /opt/software/mtl
    /opt/sources/mtl
    /opt/development/mtl
)

set(MTL_COMPILE_DEFINITIONS MTL_ASSERT_FOR_THROW=1)
foreach(feature "MOVE" "AUTO" "RANGEDFOR" "INITLIST" "STATICASSERT" "DEFAULTIMPL")
  list(APPEND MTL_COMPILE_DEFINITIONS MTL_WITH_${feature})
endforeach(feature)

include(FindPackageHandleStandardArgs)
find_package_handle_standard_args(MTL
  REQUIRED_VARS MTL_INCLUDE_DIR
)

if(MTL_FOUND AND NOT TARGET MTL::MTL)
  add_library(MTL::MTL INTERFACE IMPORTED)

  find_package(SuiteSparse QUIET)
  if (SuiteSparse_FOUND)
    list(APPEND MTL_COMPILE_DEFINITIONS "MTL_HAS_UMFPACK")
  endif (SuiteSparse_FOUND)

  set_target_properties(MTL::MTL PROPERTIES
    INTERFACE_INCLUDE_DIRECTORIES "${MTL_INCLUDE_DIR}"
    INTERFACE_COMPILE_DEFINITIONS "${MTL_COMPILE_DEFINITIONS}"
  )
endif()
