#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include "Environment.hpp"

#if HAVE_PETSC
#include <petscsys.h>
#endif

// AMDiS includes
#include <amdis/Initfile.hpp>
#include <amdis/Output.hpp>

namespace AMDiS
{
  Environment::Environment(std::string const& initFileName)
  {
    Parameters::clearData();
    if (!initFileName.empty())
      Parameters::init(initFileName);
  }


  Environment::Environment(int& argc, char**& argv, std::string const& initFileName)
    : Environment(initFileName)
  {
#if HAVE_PETSC
    PetscInitialize(&argc, &argv, PETSC_NULL, PETSC_NULL);
    petscInitialized_ = true;
#endif

    auto& mpi = Mpi::instance();
    mpi.registerMpiHelper(Dune::MPIHelper::instance(argc, argv));

    Parameters::clearData();
    if (initFileName.empty()) {
      if (argc > 1)
        Parameters::init(argv[1]);
      else
        warning("No initfile specified. Using default values for all parameters.");
    }
  }


  Environment::~Environment()
  {
#if HAVE_PETSC
    if (petscInitialized_)
      PetscFinalize();
#endif
  }

} // end namespace AMDiS
