#pragma once

#include <type_traits>
#include <dune/common/typetraits.hh>

#define AMDIS_CONCAT_IMPL( x, y ) x##y
#define AMDIS_MACRO_CONCAT( x, y ) AMDIS_CONCAT_IMPL( x, y )

#ifdef DOXYGEN
  #define REQUIRES(...)
  #define REQUIRES_(...)
  #define CONCEPT constexpr
  #define CHECK_CONCEPT(...)
#else
  #define REQUIRES(...) std::enable_if_t<__VA_ARGS__ , int> = 0
  #define REQUIRES_(...) std::enable_if_t<__VA_ARGS__ , int>
  #define CONCEPT constexpr
  #define CHECK_CONCEPT(...) static __VA_ARGS__ AMDIS_MACRO_CONCAT( _concept_check_, __COUNTER__ )
#endif

namespace AMDiS
{
  using Dune::void_t;

  namespace Concepts
  {
    namespace Impl_
    {
      template <class Concept, class = void_t<>>
      struct models
          : std::false_type
      {};

      template <class Concept, class... Ts>
      struct models<Concept(Ts...), void_t< decltype(std::declval<Concept>().requires_(std::declval<Ts>()...)) >>
          : std::true_type
      {};

      struct valid_expr
      {
        template <class... Ts>
        void operator()(Ts&&...) const;

#if defined(__GNUC__) && !defined(__clang__)
        template <class... Ts>
        void operator()(Ts const&...) const;
#endif
      };

    } // end namespace Impl_


#ifndef DOXYGEN
    template <class Concept>
    constexpr bool models = Impl_::models<Concept>::value;

    constexpr Impl_::valid_expr valid_expr = {};
#endif // DOXYGEN

  } // end namespace Concepts

} // end namespace AMDiS
