#pragma once

#include <dune/common/rangeutilities.hh>
#include <dune/functions/functionspacebases/concepts.hh>

#include <amdis/typetree/MultiIndex.hpp>
#include <amdis/utility/MappedRangeView.hpp>

namespace AMDiS
{
  /// Returns a range over (flat) DOF indices on a node, given by the localView
  template <class LocalView, class Node>
  auto nodeIndices(LocalView const& localView, Node const& node)
  {
    using namespace Dune::Functions;
    static_assert(Dune::models<Concept::LocalView<typename LocalView::GlobalBasis>, LocalView>(), "");
    static_assert(Dune::models<Concept::BasisTree<typename LocalView::GridView>, Node>(), "");

    return mappedRangeView(Dune::range(node.size()), [&](std::size_t j) -> std::size_t {
      return flatMultiIndex(localView.index(node.localIndex(j)));
    });
  }

  /// Returns a range over (flat) DOF indices on the basis tree, given by the localView
  template <class LocalView>
  auto nodeIndices(LocalView const& localView)
  {
    using namespace Dune::Functions;
    static_assert(Dune::models<Concept::LocalView<typename LocalView::GlobalBasis>, LocalView>(), "");

    return mappedRangeView(Dune::range(localView.size()), [&](std::size_t i) -> std::size_t {
      return flatMultiIndex(localView.index(i));
    });
  }


  /// Returns the number of DOF indices on a node, given by the localView
  template <class LocalView, class Node>
  std::size_t nodeIndexCount(LocalView const& /*localView*/, Node const& node)
  {
    return node.size();
  }

  /// Returns the number of DOF indices on the basis tree, given by the localView
  template <class LocalView>
  std::size_t nodeIndexCount(LocalView const& localView)
  {
    return localView.size();
  }

} // end namespace AMDiS
