#pragma once

#include <amdis/Output.hpp>

namespace AMDiS
{
  // forward declaration
  template <class RB, class CB, class T>
  class BiLinearForm;


  template <class Matrix>
  struct Constraints
  {
    template <class Mat, class Sol, class Rhs, class BitVec>
    static void dirichletBC(Mat& matrix, Sol& solution, Rhs& rhs, BitVec const& nodes, bool setDiagonal = true)
    {
      /* do nothing */
      warning("dirichletBC not implemented for this matrix type.");
    }

    template <class Mat, class Sol, class Rhs, class BitVec, class Assoc>
    static void periodicBC(Mat& matrix, Sol& solution, Rhs& rhs, BitVec const& left, Assoc const& association, bool setDiagonal = true)
    {
      /* do nothing */
      warning("periodicBC not implemented for this matrix type.");
    }
  };

  template <class Mat, class Sol, class Rhs, class BitVec>
  void dirichletBC(Mat& matrix, Sol& solution, Rhs& rhs, BitVec const& nodes, bool setDiagonal = true)
  {
    Constraints<Mat>::dirichletBC(matrix, solution, rhs, nodes, setDiagonal);
  }

  template <class Mat, class Sol, class Rhs, class BitVec, class Assoc>
  void periodicBC(Mat& matrix, Sol& solution, Rhs& rhs, BitVec const& left, Assoc const& association, bool setDiagonal = true)
  {
    Constraints<Mat>::periodicBC(matrix, solution, rhs, left, association, setDiagonal);
  }


  template <class RB, class CB, class T>
  struct Constraints<BiLinearForm<RB,CB,T>>
  {
    using Matrix = BiLinearForm<RB,CB,T>;

    template <class Sol, class Rhs, class BitVec>
    static void dirichletBC(Matrix& matrix, Sol& solution, Rhs& rhs, BitVec const& nodes, bool setDiagonal = true)
    {
      AMDiS::dirichletBC(matrix.backend(), solution.backend(), rhs.backend(), nodes, setDiagonal);
    }

    template <class Sol, class Rhs, class BitVec, class Assoc>
    static void periodicBC(Matrix& matrix, Sol& solution, Rhs& rhs, BitVec const& left, Assoc const& association, bool setDiagonal = true)
    {
      AMDiS::periodicBC(matrix.backend(), solution.backend(), rhs.backend(), left, association, setDiagonal);
    }
  };

} // end namespace AMDiS
