#pragma once

#include <Eigen/SparseCore>

#include <dune/grid/common/partitionset.hh>
#include <amdis/linearalgebra/Communication.hpp>

namespace AMDiS
{
  /** Traits class for a linear solver for the system AX=B using an FE space described by a dune-functions Basis
   *  Contains typedefs specific to the EIGEN backend.
   */
  template <class Basis, class T = double>
  struct BackendTraits
  {
    using Mat = Eigen::SparseMatrix<T, Eigen::RowMajor>;
    using Vec = Eigen::Matrix<T, Eigen::Dynamic, 1>;
    using CoefficientType = T;
    using Comm = SequentialCommunication;
    using PartitionSet = Dune::Partitions::All;
  };

} // end namespace AMDiS
