#pragma once

#include <memory>

#include <dune/istl/preconditioner.hh>
#include <dune/istl/solver.hh>

namespace AMDiS
{
  template <class Traits>
  struct ISTLPreconCreatorInterface
  {
    using Mat = typename Traits::Mat;
    using Vec = typename Traits::Vec;
    using Comm = typename Traits::Comm;

    virtual ~ISTLPreconCreatorInterface() = default;

    using Interface = Dune::Preconditioner<Vec, Vec>;
    virtual std::unique_ptr<Interface> create(Mat const& mat, Comm const& comm) const = 0;
  };

  template <class Traits>
  struct ISTLSolverCreatorInterface
  {
    using Mat = typename Traits::Mat;
    using Vec = typename Traits::Vec;
    using Comm = typename Traits::Comm;

    virtual ~ISTLSolverCreatorInterface() = default;

    using Interface = Dune::InverseOperator<Vec, Vec>;
    virtual std::unique_ptr<Interface> create(Mat const& mat, Comm const& comm) const = 0;
  };

} // end namespace AMDiS