#pragma once

#include <amdis/linearalgebra/PreconditionerInterface.hpp>

namespace AMDiS
{
  /**
   * \ingroup Solver
   *
   * \brief Wrapper for using ITL preconditioners in AMDiS.
   */
  template <class Traits, class PreconRunner>
  class Preconditioner
      : public PreconditionerInterface<Traits>
  {
    using Self = Preconditioner;
    using Matrix = typename Traits::Mat;
    using Vector = typename Traits::Vec;
    using Super = PreconditionerInterface<Traits>;

  public:
    /// A creator to be used instead of the constructor.
    struct Creator : CreatorInterface<Super>
    {
      std::unique_ptr<Super> create() override
      {
        return std::make_unique<Self>();
      }
    };

  public:
    /// Implementation of \ref PreconditionerBase::init()
    void init(Matrix const& fullMatrix) override
    {
      precon_.reset(new PreconRunner(fullMatrix));
    }

    /// Implementation of \ref PreconditionerInterface::exit()
    void exit() override
    {
      precon_.reset();
    }

    /// Implementation of \ref PreconditionerBase::solve()
    void solve(Vector const& vin, Vector& vout) const override
    {
      test_exit_dbg(bool(precon_), "No preconditioner initialized!");
      precon_->solve(vin, vout);
    }

    /// Implementation of \ref PreconditionerBase::adjoint_solve()
    void adjoint_solve(Vector const& vin, Vector& vout) const override
    {
      test_exit_dbg(bool(precon_), "No preconditioner initialized!");
      precon_->adjoint_solve(vin, vout);
    }

  private:
    std::shared_ptr<PreconRunner> precon_;
  };

} // namespace AMDiS
