#pragma once

#include <vector>

#include <petscksp.h>

#include <amdis/CreatorMap.hpp>
#include <amdis/linearalgebra/LinearSolver.hpp>
#include <amdis/linearalgebra/petsc/PetscRunner.hpp>

namespace AMDiS
{
  /// Adds default creators for linear solvers for the Petsc backend.
  template <class Traits>
  class DefaultCreators< LinearSolverInterface<Traits> >
  {
    using Solver = typename LinearSolver<Traits, PetscRunner<Traits>>::Creator;
    using Map = CreatorMap<LinearSolverInterface<Traits>>;

  public:
    static void init()
    {
      // see https://www.mcs.anl.gov/petsc/petsc-current/docs/manualpages/KSP/KSPType.html
      static const std::vector<std::string> kspTypes_ = {
        KSPCG, KSPFCG, KSPGMRES, KSPFGMRES, KSPPREONLY, KSPMINRES, KSPCGS, KSPBCGS, KSPCGNE
#ifdef KSPCHEBYSHEV
        , KSPCHEBYSHEV
#endif
#ifdef KSPGROPPCG
        , KSPGROPPCG
#endif
#ifdef KSPPIPECG
        , KSPPIPECG
#endif
#ifdef KSPPIPECGRR
        , KSPPIPECGRR
#endif
#ifdef KSPPIPELCG
        , KSPPIPELCG
#endif
#ifdef KSPCGNASH
        , KSPCGNASH
#endif
#ifdef KSPCGSTCG
        , KSPCGSTCG
#endif
#ifdef KSPCGGLTR
        , KSPCGGLTR
#endif
#ifdef KSPPIPEFCG
        , KSPPIPEFCG
#endif
#ifdef KSPPIPEFGMRES
        , KSPPIPEFGMRES
#endif
#ifdef KSPLGMRES
        , KSPLGMRES
#endif
#ifdef KSPDGMRES
        , KSPDGMRES
#endif
#ifdef KSPPGMRES
        , KSPPGMRES
#endif
#ifdef KSPTCQMR
        , KSPTCQMR
#endif
#ifdef KSPIBCGS
        , KSPIBCGS
#endif
#ifdef KSPFBCGS
        , KSPFBCGS
#endif
#ifdef KSPFBCGSR
        , KSPFBCGSR
#endif
#ifdef KSPBCGSL
        , KSPBCGSL
#endif
#ifdef KSPPIPEBCGS
        , KSPPIPEBCGS
#endif
#ifdef KSPTFQMR
        , KSPTFQMR
#endif
#ifdef KSPCR
        , KSPCR
#endif
#ifdef KSPPIPECR
        , KSPPIPECR
#endif
#ifdef KSPLSQR
        , KSPLSQR
#endif
#ifdef KSPQCG
        , KSPQCG
#endif
#ifdef KSPBICG
        , KSPBICG
#endif
#ifdef KSPSYMMLQ
        , KSPSYMMLQ
#endif
#ifdef KSPLCD
        , KSPLCD
#endif
#ifdef KSPPYTHON
        , KSPPYTHON
#endif
#ifdef KSPGCR
        , KSPGCR
#endif
#ifdef KSPPIPEGCR
        , KSPPIPEGCR
#endif
#ifdef KSPTSIRM
        , KSPTSIRM
#endif
#ifdef KSPCGLS
        , KSPCGLS
#endif
#ifdef KSPFETIDP
        , KSPFETIDP
#endif
#ifdef KSPRICHARDSON
        , KSPRICHARDSON
#endif
      };

      auto solver = new Solver;
      Map::addCreator("default", solver);
      Map::addCreator("direct", solver);
      for (auto const& kspType : kspTypes_)
        Map::addCreator(kspType, solver);
    }
  };

} // end namespace AMDiS
