#pragma once

#include <type_traits>

#include <petscmat.h>
#include <petscvec.h>

#include <dune/grid/common/partitionset.hh>
#include <amdis/linearalgebra/petsc/Communication.hpp>

namespace AMDiS
{
  /** Traits class for a linear solver for the system AX=B using an FE space described by a dune-functions Basis
   *  Contains typedefs specific to the PETSc backend.
   */
  template <class Basis, class T = double>
  struct BackendTraits
  {
    static_assert(std::is_same<T,PetscScalar>::value, "");

    using Mat = ::Mat;
    using Vec = ::Vec;
    using CoefficientType = PetscScalar;
    using Comm = DistributedCommunication<Basis>;
    using PartitionSet = Dune::Partitions::Interior;
  };

} // end namespace AMDiS
