#pragma once

#include <type_traits>

#include <amdis/localoperators/FirstOrderTestGradTrial.hpp>

namespace AMDiS
{
  /**
   * \addtogroup operators
   * @{
   **/

  namespace tag
  {
    struct gradtest_trial {};
    struct grad_test {};
  }


  /// first-order operator \f$ \langle\nabla\psi, \mathbf{b}\,\phi\rangle \f$
  template <class LC, class GridFct>
  class GridFunctionOperator<tag::gradtest_trial, LC, GridFct>
      : public GridFunctionOperatorTransposed<GridFunctionOperator<tag::gradtest_trial, LC, GridFct>,
                                              GridFunctionOperator<tag::test_gradtrial, LC, GridFct>>
  {
    using Self = GridFunctionOperator;
    using Transposed = GridFunctionOperator<tag::test_gradtrial, LC, GridFct>;
    using Super = GridFunctionOperatorTransposed<Self, Transposed>;

  public:
    GridFunctionOperator(tag::gradtest_trial, GridFct const& expr)
      : Super(tag::test_gradtrial{}, expr)
    {}
  };


  /// Create a first-order term with derivative on trial-function
  template <class Expr, class... QuadratureArgs>
  auto fot(Expr&& expr, tag::grad_test, QuadratureArgs&&... args)
  {
    return makeOperator(tag::gradtest_trial{}, FWD(expr), FWD(args)...);
  }

  /** @} **/

} // end namespace AMDiS
