#pragma once

#include <type_traits>

#include <amdis/GridFunctionOperator.hpp>
#include <amdis/utility/LocalBasisCache.hpp>
#include <amdis/common/StaticSize.hpp>

namespace AMDiS
{
  /**
   * \addtogroup operators
   * @{
   **/

  namespace tag
  {
    struct test {};
  }


  /// zero-order vector-operator \f$ (c\, \psi) \f$
  template <class LC, class GridFct>
  class GridFunctionOperator<tag::test, LC, GridFct>
      : public GridFunctionOperatorBase<GridFunctionOperator<tag::test, LC, GridFct>, LC, GridFct>
  {
    using Self = GridFunctionOperator;
    using Super = GridFunctionOperatorBase<Self, LC, GridFct>;

    static_assert( Size_v<typename GridFct::Range> == 1, "Expression must be of scalar type." );

  public:
    GridFunctionOperator(tag::test, GridFct const& expr)
      : Super(expr, 0)
    {}

    template <class CG, class Node, class Vec>
    void getElementVector(CG const& contextGeo, Node const& node, Vec& elementVector)
    {
      static_assert(Node::isLeaf, "Operator can be applied to Leaf-Nodes only");

      auto const& quad = this->getQuadratureRule(contextGeo.type(), node);
      auto const& localFE = node.finiteElement();
      std::size_t size = localFE.size();

      NodeQuadCache<Node> cache(localFE.localBasis());

      auto const& shapeValuesCache = cache.evaluateFunctionAtQP(contextGeo,quad);
      for (std::size_t iq = 0; iq < quad.size(); ++iq) {
        // Position of the current quadrature point in the reference element
        decltype(auto) local = contextGeo.local(quad[iq].position());

        // The multiplicative factor in the integral transformation formula
        auto factor = contextGeo.integrationElement(quad[iq].position()) * quad[iq].weight();
        factor *= Super::coefficient(local);

        auto const& shapeValues = shapeValuesCache[iq];
        for (std::size_t i = 0; i < size; ++i) {
          const auto local_i = node.localIndex(i);
          elementVector[local_i] += factor * shapeValues[i];
        }
      }
    }
  };

  /** @} **/

} // end namespace AMDiS
