#pragma once

#include <cmath>

#include <dune/common/typetraits.hh>
#include <dune/typetree/nodetags.hh>

#include <amdis/common/ForEach.hpp>
#include <amdis/common/Index.hpp>
#include <amdis/common/Tags.hpp>

namespace AMDiS
{
  namespace Impl
  {
    template <class Node, class NodeTag>
    struct FiniteElementTypeImpl
    {
      static_assert( Dune::AlwaysFalse<NodeTag>::value, "Unknown node-type for range definition" );

      // polynomial degree of finite-element basis functions
      static int order(Node const&) { return 0; }
    };
  }

  template <class Node>
  using FiniteElementType =
    Impl::FiniteElementTypeImpl<Node, typename Node::NodeTag>;

  template <class Node>
  using FiniteElementType_t = typename FiniteElementType<Node>::type;

  template <class Node>
  int polynomialDegree(Node const& node)
  {
    return FiniteElementType<Node>::order(node);
  }


  namespace Impl
  {
    // Leaf node
    template <class Node>
    struct FiniteElementTypeImpl<Node, Dune::TypeTree::LeafNodeTag>
    {
      using type = typename Node::FiniteElement;

      static int order(Node const& node)
      {
        return node.finiteElement().localBasis().order();
      }
    };

    // Power node
    template <class Node>
    struct FiniteElementTypeImpl<Node, Dune::TypeTree::PowerNodeTag>
    {
      using ChildNode = typename Node::template Child<0>::type;
      using type = FiniteElementType_t<ChildNode>;

      static int order(Node const& node)
      {
        return FiniteElementType<ChildNode>::order(node.child(0));
      }
    };

    // Composite node
    template <class Node>
    struct FiniteElementTypeImpl<Node, Dune::TypeTree::CompositeNodeTag>
    {
      using type = tag::unknown; // no common FiniteElement type

      static int order(Node const& node)
      {
        int degree = 0;
        Tools::for_range<0,Node::CHILDREN>([&](auto const _i) {
          degree = std::max(degree, polynomialDegree(node.child(_i)));
        });
        return degree;
      }
    };

  } // end namespace Impl

} // end namespace AMDiS
