#include <amdis/AMDiS.hpp>
#include <amdis/common/HybridSize.hpp>
#include <amdis/common/TypeTraits.hpp>

#include "Tests.hpp"

using namespace AMDiS;

struct Vec1
{
  static const std::size_t dimension = 1;

  int operator[](std::size_t) const { return 0; }
  std::size_t size() const { return 2; }
};

struct Vec2
{
  static const std::size_t dimension = 1;

  template <std::size_t I>
  int operator[](index_t<I>) const { return 0; }
  std::size_t size() const { return 2; }
};

struct Mat1
{
  int operator()(std::size_t,std::size_t) const { return 0; }
  std::size_t N() const { return 2; }
  std::size_t M() const { return 2; }
};

struct Mat2
{
  int operator()(std::size_t,std::size_t) const { return 0; }
  std::size_t rows() const { return 2; }
  std::size_t cols() const { return 2; }
};

struct Mat3
{
  int operator()(std::size_t,std::size_t) const { return 0; }
  std::size_t num_rows() const { return 2; }
  std::size_t num_cols() const { return 2; }
};

struct Mat4
{
  static const std::size_t rows = 1;
  static const std::size_t cols = 1;

  template <std::size_t I, std::size_t J>
  int operator()(index_t<I>, index_t<J>) const { return 0; }
};

struct NotAccessible {};

int main(int argc, char** argv)
{
  Environment env(argc, argv);

  Vec1 vec1;
  Vec2 vec2;

  AMDIS_TEST_EQ(std::size_t(hybridSize(vec1)),2);
  AMDIS_TEST_EQ(std::size_t(hybridSize(vec2)),1);

  auto s = hybridSize(vec2);
  static_assert(VALUE(s) == 1, "");

  Mat1 mat1;
  Mat2 mat2;
  Mat3 mat3;
  Mat4 mat4;

  AMDIS_TEST_EQ(std::size_t(hybridNumRows(mat1)),2);
  AMDIS_TEST_EQ(std::size_t(hybridNumCols(mat1)),2);

  AMDIS_TEST_EQ(std::size_t(hybridNumRows(mat2)),2);
  AMDIS_TEST_EQ(std::size_t(hybridNumCols(mat2)),2);

  AMDIS_TEST_EQ(std::size_t(hybridNumRows(mat3)),2);
  AMDIS_TEST_EQ(std::size_t(hybridNumCols(mat3)),2);

  AMDIS_TEST_EQ(std::size_t(hybridNumRows(mat4)),1);
  AMDIS_TEST_EQ(std::size_t(hybridNumCols(mat4)),1);

  auto r = hybridNumRows(mat4);
  auto c = hybridNumCols(mat4);
  static_assert(VALUE(r) == 1, "");
  static_assert(VALUE(c) == 1, "");

  return 0;
}
