#include <amdis/common/FieldMatVec.hpp>
#include <amdis/common/MultiTypeVector.hpp>

#include "Tests.hpp"

using namespace AMDiS;
using namespace Dune::Indices;

int main()
{
  using F2 = FieldVector<double,2>;
  using F3 = FieldVector<double,3>;
  using V = MultiTypeVector<F2, F3>;

  // constructors
  V v0{}; // default constructor

  F2 f2 = {1.0, 2.0};
  F3 f3 = {1.0, 2.0, 3.0};

//  V tmp = {F2{1.0, 2.0}, F3{1.0, 2.0, 3.0}};

  V v1{f2,f3}; // constructor with FV arguments
  AMDIS_TEST_EQ( v1[_0] , f2 );
  AMDIS_TEST_EQ( v1[_1] , f3 );

  V v2{v1}; // copy constructor
  AMDIS_TEST_EQ( v2[_0] , f2 );
  AMDIS_TEST_EQ( v2[_1] , f3 );

  v0 = v1; // copy assignment
  AMDIS_TEST_EQ( v0[_0] , f2 );
  AMDIS_TEST_EQ( v0[_1] , f3 );

  V v3{0.0}; // construction from constant
  AMDIS_TEST_EQ( v3[_0] , F2({0.0, 0.0}) );
  AMDIS_TEST_EQ( v3[_1] , F3({0.0, 0.0, 0.0}) );

  v3 = 1.0; // assignment of constant
  AMDIS_TEST_EQ( v3[_0] , F2({1.0, 1.0}) );
  AMDIS_TEST_EQ( v3[_1] , F3({1.0, 1.0, 1.0}) );

  // compound assignment operators
  v2 += v1;
  AMDIS_TEST_EQ( v2[_0] , F2({2.0, 4.0}) );
  AMDIS_TEST_EQ( v2[_1] , F3({2.0, 4.0, 6.0}) );
  v2 -= v1;

  v2 *= 1.5;
  AMDIS_TEST_EQ( v2[_0] , F2({1.5, 3.0}) );
  AMDIS_TEST_EQ( v2[_1] , F3({1.5, 3.0, 4.5}) );
  v2 /= 1.5;

  v1[_1] += f3;
  AMDIS_TEST( v1[_1] == F3({2.0, 4.0, 6.0}) );

  {
    V v0_tmp;
    V v1_tmp{std::move(v0_tmp)}; // move constructor
    v1 = std::move(v1_tmp); // move assignment
  }


  // element access
  v1[_0] = f2; // mutable block access
  v1[_0][0] = 2.5; // mutable element access
  AMDIS_TEST( v1[_0][0] == 2.5 && v1[_0][1] == 2.0);
  DUNE_UNUSED auto x = v1[_1][1]; // const access

  // multi-index access
  std::array<std::size_t,2> index{1,1};
  v1[index] = 2.0;
  AMDIS_TEST( v1[_1][1] == 2.0 );

  using FV = FieldVector<double,1>;
  using W0 = MultiTypeVector<FV,FV>;
  using W = MultiTypeVector<W0,W0>;
  W w;
  w[index] = 3.0;
  AMDIS_TEST( w[_1][_1] == 3.0 );

  // dimension of the tuple
  AMDIS_TEST( v1.size() == 2 );
}