// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
// vi: set et ts=4 sw=2 sts=2:
#include "config.h"
#include <amdis/AMDiS.hpp>
#include <amdis/ProblemStat.hpp>
#include <amdis/LocalOperators.hpp>
#include <amdis/localoperators/ConvectionDiffusionOperator.hpp>
#include <amdis/localoperators/StokesOperator.hpp>

using namespace AMDiS;

using Grid = Dune::YaspGrid<2>;
using Param   = TaylorHoodBasis<Grid>;
using Problem = ProblemStat<Param>;

int main(int argc, char** argv)
{
  Environment env(argc, argv);

  using namespace Dune::Indices;

  Problem prob("ellipt");
  prob.initialize(INIT_ALL);

  auto _v = _0;
  auto _p = _1;

  FieldVector<double,1> scalar = 1.0;
  FieldVector<double,2> vec; vec = 1.0;
  FieldMatrix<double,2,2> mat; mat = 1.0;

  auto op1 = makeOperator(tag::divtestvec{}, scalar);
  prob.addVectorOperator(op1, _v);

  auto op2 = makeOperator(tag::divtestvec_trial{}, scalar);
  prob.addMatrixOperator(op2, _v, _p);

  auto op3 = makeOperator(tag::gradtest{}, vec);
  prob.addVectorOperator(op3, _p);

  auto op4 = makeOperator(tag::gradtest_trial{}, vec);
  prob.addMatrixOperator(op4, _p, _p);

  auto op5a = makeOperator(tag::gradtest_trialvec{}, scalar);
  // auto op5b = makeOperator(tag::gradtest_trialvec{}, mat); // TODO: needs to be implemented
  prob.addMatrixOperator(op5a, _p, _v);

  auto op6 = makeOperator(tag::partialtest{0}, scalar);
  prob.addVectorOperator(op6, _p);

  auto op7 = makeOperator(tag::partialtest_trial{0}, scalar);
  prob.addMatrixOperator(op7, _p, _p);

  auto op8 = makeOperator(tag::test_divtrialvec{}, scalar);
  prob.addMatrixOperator(op8, _p, _v);

  auto op9 = makeOperator(tag::test_gradtrial{}, vec);
  prob.addMatrixOperator(op9, _p, _p);

  auto op10 = makeOperator(tag::test_partialtrial{0}, scalar);
  prob.addMatrixOperator(op10, _p, _p);

  auto op11a = makeOperator(tag::testvec_gradtrial{}, scalar);
  // auto op11b = makeOperator(tag::testvec_gradtrial{}, mat); // TODO: needs to be implemented
  prob.addMatrixOperator(op11a, _v, _p);

  auto op12 = makeOperator(tag::divtestvec_divtrialvec{}, scalar);
  prob.addMatrixOperator(op12, _v, _v);

  auto op13a = makeOperator(tag::gradtest_gradtrial{}, scalar);
  auto op13b = makeOperator(tag::gradtest_gradtrial{}, mat);
  prob.addMatrixOperator(op13a, _p, _p);
  prob.addMatrixOperator(op13b, _p, _p);

  auto op14 = makeOperator(tag::partialtest_partialtrial{0,0}, scalar);
  prob.addMatrixOperator(op14, _p, _p);

  auto op15 = makeOperator(tag::test{}, scalar);
  prob.addVectorOperator(op15, _p);

  auto op16 = makeOperator(tag::test_trial{}, scalar);
  prob.addMatrixOperator(op16, _p, _p);

  auto op17 = makeOperator(tag::test_trialvec{}, vec);
  prob.addMatrixOperator(op17, _p, _v);

  auto op18 = makeOperator(tag::testvec{}, vec);
  prob.addVectorOperator(op18, _v);

  auto op19 = makeOperator(tag::testvec_trial{}, vec);
  prob.addMatrixOperator(op19, _v, _p);

  auto op20a = makeOperator(tag::testvec_trialvec{}, scalar);
  auto op20b = makeOperator(tag::testvec_trialvec{}, mat);
  prob.addMatrixOperator(op20a, _v, _v);
  prob.addMatrixOperator(op20b, _v, _v);

  // some special operators

  auto opStokes = makeOperator(tag::stokes{}, scalar);
  prob.addMatrixOperator(opStokes);
  prob.addMatrixOperator(opStokes, {}, {});

  auto opCDa = convectionDiffusion(scalar, scalar, scalar, scalar);
  prob.addMatrixOperator(opCDa, _p, _p);
  prob.addVectorOperator(opCDa, _p);

  auto opCDb = convectionDiffusion(mat, vec, scalar, scalar, std::false_type{});
  prob.addMatrixOperator(opCDb, _p, _p);
  prob.addVectorOperator(opCDb, _p);

  return 0;
}
